# Please keep this package sync with FC
#
%bcond_with snapshot_build

%if %{with snapshot_build}
# Unlock LLVM Snapshot LUA functions
%{llvm_sb_verbose}
%{llvm_sb}
%endif

%global major 17
%define libname %mklibname lld %major
%define devname %mklibname -d lld

%global toolchain clang

# Opt out of https://fedoraproject.org/wiki/Changes/fno-omit-frame-pointer
# https://bugzilla.redhat.com/show_bug.cgi?id=2158587
%undefine _include_frame_pointers

%bcond_without check
%bcond_with compat_build

%global maj_ver 17
%global min_ver 0
%global patch_ver 6
#global rc_ver 4

%if %{with snapshot_build}
%undefine rc_ver
%global maj_ver %{llvm_snapshot_version_major}
%global min_ver %{llvm_snapshot_version_minor}
%global patch_ver %{llvm_snapshot_version_patch}
%endif

%global lld_version %{maj_ver}.%{min_ver}.%{patch_ver}

%global lld_srcdir lld-%{lld_version}%{?rc_ver:rc%{rc_ver}}.src

%if %{with compat_build}
%global pkg_name lld%{maj_ver}
%global install_prefix %{_libdir}/llvm%{maj_ver}
%global install_includedir %{install_prefix}/include
%global install_libdir %{install_prefix}/lib
%global install_datadir %{install_prefix}/share
%else
%global pkg_name lld
%global install_prefix /usr
%global install_includedir %{_includedir}
%global install_libdir %{_libdir}
%global install_datadir %{_datadir}
%endif

%define rel 3
%global baserelease %{rel}

Name:		%{pkg_name}
Version:	%{maj_ver}.%{min_ver}.%{patch_ver}
Release:	%mkrel %{?rc_ver:0.rc%{rc_ver}}%{baserelease}
Summary:	The LLVM Linker
Group:		Development/Tools

License:	Apache-2.0 WITH LLVM-exception OR NCSA
URL:		http://llvm.org
%if %{with snapshot_build}
Source0:	%{llvm_snapshot_source_prefix}lld-%{llvm_snapshot_yyyymmdd}.src.tar.xz
%{llvm_snapshot_extra_source_tags}
%else
Source0:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{maj_ver}.%{min_ver}.%{patch_ver}%{?rc_ver:-rc%{rc_ver}}/%{lld_srcdir}.tar.xz
Source1:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{maj_ver}.%{min_ver}.%{patch_ver}%{?rc_ver:-rc%{rc_ver}}/%{lld_srcdir}.tar.xz.sig
Source2:	release-keys.asc
%endif

ExcludeArch:	s390x

# Bundle libunwind header need during build for MachO support
Patch1:		0002-PATCH-lld-Import-compact_unwind_encoding.h-from-libu.patch

BuildRequires:	clang
BuildRequires:	cmake
BuildRequires:	ninja-build
%if %{with compat_build}
BuildRequires:	llvm%{maj_ver}-devel = %{version}
BuildRequires:	llvm%{maj_ver}-cmake-utils = %{version}
%else
BuildRequires:	llvm-devel = %{version}
BuildRequires:	llvm-cmake-utils = %{version}
BuildRequires:	llvm-test = %{version}
BuildRequires:	llvm-googletest = %{version}
%endif
BuildRequires:	ncurses-devel
BuildRequires:	zlib-devel

# For make check:
BuildRequires:	python3-rpm-macros
BuildRequires:	python3-lit

# For gpg source verification
BuildRequires:	gnupg2

# For /sbin/alternatives:
Requires(post): update-alternatives
Requires(preun): update-alternatives

Requires: %{name}-libs = %{version}-%{release}

%description
The LLVM project linker.

%package -n %devname
Group: Development/C
Summary:	Libraries and header files for LLD
Requires: %{name}-libs%{?_isa} = %{version}-%{release}
%if %{without compat_build}
# lld tools are referenced in the cmake files, so we need to add lld as a
# dependency.
Requires: %{name}%{?_isa} = %{version}-%{release}
%endif
Provides: %{name}-devel = %{version}-%{release}
Provides: %{name}-devel%{?_isa} = %{version}-%{release}
Provides: %{name}-devel(major) = %{maj_ver}

%description -n %devname
This package contains library and header files needed to develop new native
programs that use the LLD infrastructure.

%package -n %libname
Summary:	LLD shared libraries
Group: System/Libraries
Provides: %{name}-libs = %{version}-%{release}
Provides: %{name}-libs%{?_isa} = %{version}-%{release}

%description -n %libname
Shared libraries for LLD.

%prep
%if %{without snapshot_build}
%{gpgverify} --keyring='%{SOURCE2}' --signature='%{SOURCE1}' --data='%{SOURCE0}'
%endif

%autosetup -n %{lld_srcdir} -p2

%if %{with compat_build}
# For compat builds, we don't want to build the actual lld binary. While there is an
# LLD_BUILD_TOOLS cmake option, it is incomplete in various ways (e.g. still leaves install
# targets and symlinks), so instead skip the tools/lld build entirely.
# We can't simply delete the binaries after the fact, because this would leave checks for
# their existence in the cmake exports.
sed 's/add_subdirectory(tools\/lld)//' -i CMakeLists.txt
%endif

%build
# FC compatibility:
%global _vpath_builddir %{_target}-build

%cmake \
	-GNinja \
	-DCMAKE_BUILD_TYPE=RelWithDebInfo \
	-DCMAKE_INSTALL_PREFIX=%{install_prefix} \
	-DLLVM_LINK_LLVM_DYLIB:BOOL=ON \
	-DLLVM_DYLIB_COMPONENTS="all" \
	-DLLVM_COMMON_CMAKE_UTILS=%{install_datadir}/llvm/cmake \
	-DCMAKE_SKIP_RPATH:BOOL=ON \
	-DPYTHON_EXECUTABLE=%{__python3} \
%if %{with compat_build}
	-DLLVM_CMAKE_DIR=%{install_libdir}/cmake/llvm \
	-DLLVM_INCLUDE_TESTS=OFF \
%else
	-DLLVM_INCLUDE_TESTS=ON \
	-DLLVM_EXTERNAL_LIT=%{_bindir}/lit \
	-DLLVM_LIT_ARGS="-sv \
	--path %{_libdir}/llvm" \
%if %{with snapshot_build}
	-DLLVM_VERSION_SUFFIX="%{llvm_snapshot_version_suffix}" \
%endif
%if 0%{?__isa_bits} == 64
	-DLLVM_LIBDIR_SUFFIX=64 \
%else
	-DLLVM_LIBDIR_SUFFIX= \
%endif
%endif
	-DLLVM_MAIN_SRC_DIR=%{_datadir}/llvm/src

%cmake_build

%install

# Install libraries and binaries
%cmake_install

# This is generated by Patch1 during build and (probably) must be removed afterward
rm %{buildroot}%{install_includedir}/mach-o/compact_unwind_encoding.h

%if %{without compat_build}
# Required when using update-alternatives:
# https://docs.fedoraproject.org/en-US/packaging-guidelines/Alternatives/
touch %{buildroot}%{_bindir}/ld

install -D -m 644 -t  %{buildroot}%{_mandir}/man1/ docs/ld.lld.1

%post
%{_sbindir}/update-alternatives --install %{_bindir}/ld ld %{_bindir}/ld.lld 1

%postun
if [ $1 -eq 0 ] ; then
  %{_sbindir}/update-alternatives --remove ld %{_bindir}/ld.lld
fi
%endif

%check

%if %{without compat_build}
# armv7lhl tests disabled because of arm issue, see https://koji.fedoraproject.org/koji/taskinfo?taskID=33660162
%ifnarch %{arm}
%if %{with check}
%cmake_build --target check-lld
%endif

%endif

%endif

%if %{without compat_build}
%files
%license LICENSE.TXT
%ghost %{_bindir}/ld
%{_bindir}/lld*
%{_bindir}/ld.lld
%{_bindir}/ld64.lld
%{_bindir}/wasm-ld
%{_mandir}/man1/ld.lld.1*
%endif

%files -n %devname
%{install_includedir}/lld
%{install_libdir}/liblld*.so
%{install_libdir}/cmake/lld/

%files -n %libname
%{install_libdir}/liblld*.so.%{major}



%changelog
* Thu Apr 11 2024 wally <wally> 17.0.6-3.mga10
+ Revision: 2056619
- build using fixed clang on all arches

* Thu Apr 11 2024 wally <wally> 17.0.6-2.mga10
+ Revision: 2056496
- build i686 with gcc toolchain for now

* Mon Feb 26 2024 wally <wally> 17.0.6-1.mga10
+ Revision: 2044483
- new version 17.0.6 (Sync with Fedora)

* Fri Dec 09 2022 tv <tv> 15.0.6-1.mga9
+ Revision: 1919676
- Update to LLVM 15.0.6

* Tue Nov 15 2022 tv <tv> 15.0.4-2.mga9
+ Revision: 1907457
- Update to LLVM 15.0.4

* Wed Oct 12 2022 tv <tv> 15.0.0-2.mga9
+ Revision: 1896538
- Remove lld-test package

* Wed Sep 21 2022 tv <tv> 15.0.0-1.mga9
+ Revision: 1891058
- Update to LLVM 15.0.0
- Set build type to RelWithDebInfo

* Fri Jun 17 2022 tv <tv> 14.0.5-1.mga9
+ Revision: 1864686
- Update to 14.0.5

* Wed Mar 30 2022 tv <tv> 14.0.0-1.mga9
+ Revision: 1834553
- Update to 14.0.0rc1

* Thu Feb 03 2022 tv <tv> 13.0.1-0.1.rc3.mga9
+ Revision: 1771213
- Update to 13.0.1 final

* Wed Oct 20 2021 tv <tv> 13.0.0-1.mga9
+ Revision: 1752284
- Disable testsuite for now
- Define missing %%__cmake_builddir macro
- 13.0.0 Release
- Add --without=check option

* Mon Jul 19 2021 tv <tv> 12.0.1-1.mga9
+ Revision: 1737042
- 12.0.1 Release

* Fri Apr 16 2021 tv <tv> 12.0.0-1.mga9
+ Revision: 1715991
- 12.0.0 final

* Fri Apr 09 2021 tv <tv> 12.0.0-0.8.rc5.mga9
+ Revision: 1714256
- 12.0.0 RC5
- Set executable permissions on run-lit-tests

* Fri Apr 02 2021 tv <tv> 12.0.0-0.4.rc4.mga9
+ Revision: 1712623
- 12.0.0 RC4

* Fri Mar 12 2021 tv <tv> 12.0.0-0.4.rc3.mga9
+ Revision: 1701746
- LLVM 12.0.0 rc3

* Sun Feb 28 2021 tv <tv> 12.0.0-0.2.rc2.mga9
+ Revision: 1693219
- llvm 12.0.0-rc2 release

* Thu Jan 07 2021 tv <tv> 11.0.1-3.2.mga8
+ Revision: 1669445
- LLVM 11.0.1 final

* Wed Dec 23 2020 tv <tv> 11.0.1-2.rc2.2.mga8
+ Revision: 1663058
- llvm 11.0.1-rc2

* Wed Dec 02 2020 tv <tv> 11.0.1-1.rc1.2.mga8
+ Revision: 1651722
- llvm 11.0.1-rc1

* Wed Nov 25 2020 tv <tv> 11.0.0-3.2.mga8
+ Revision: 1649295
- 11.0.0

* Fri Oct 16 2020 ghibo <ghibo> 10.0.1-2.mga8
+ Revision: 1636367
- Rebuild to fix weird package release number

* Thu Oct 08 2020 akien <akien> 10.0.1-1.2.mga8
+ Revision: 1632801
- Version 10.0.1

* Thu Sep 10 2020 tv <tv> 10.0.0-3.2.mga8
+ Revision: 1624176
- fix file deps

* Mon Jul 20 2020 tv <tv> 10.0.0-3.1.mga8
+ Revision: 1607465
- adjust for %%cmake_install being different from FC's one
- adjust for %%cmake_build being different from FC's one
- Use generic cmake macros
- Use Ninja as build system
- Remove chrpath dependency

* Mon Jul 20 2020 tv <tv> 10.0.0-2.mga8
+ Revision: 1607425
- Make test archive arch-independent

* Tue Mar 24 2020 tv <tv> 10.0.0-1.mga8
+ Revision: 1559499
- 10.0.0 final

* Thu Mar 19 2020 tv <tv> 10.0.0-0.5.rc5.mga8
+ Revision: 1558095
- 10.0.0 rc5

* Sun Mar 15 2020 tv <tv> 10.0.0-0.4.rc4.mga8
+ Revision: 1556555
- 10.0.0 rc4
- enable verifying sources with latest rpm-mageia-setup

* Thu Mar 05 2020 tv <tv> 10.0.0-0.3.rc3.mga8
+ Revision: 1554130
- 10.0.0 RC3

* Thu Mar 05 2020 tv <tv> 10.0.0-0.2.rc2.mga8
+ Revision: 1554127
- 10.0.0 RC3
- 10.0.0 RC2

* Thu Feb 13 2020 tv <tv> 10.0.0-0.1.1.rc1.mga8
+ Revision: 1512927
- fix deps

* Wed Feb 12 2020 tv <tv> 10.0.0-0.1.rc1.mga8
+ Revision: 1510128
- disable brp-python-bytecompile automagic
- 10.0.0 rc1
+ wally <wally>
- build with new cmake macros

* Sat Sep 21 2019 tv <tv> 9.0.0-1.mga8
+ Revision: 1444191
- 9.0.0 Release

* Tue Sep 17 2019 tv <tv> 9.0.0-0.1.rc3.mga8
+ Revision: 1442776
- drop BR on python2-lit
- 9.0.0-rc3 Release
- touch /usr/bin/ld as required by the packaging guidelines for
  update-alternatives
- Add update-alternative for ld

* Sat Apr 13 2019 tv <tv> 8.0.0-2.mga7
+ Revision: 1389641
- 8.0.0
- ignore test failure for now

* Fri Feb 01 2019 tv <tv> 7.0.1-2.mga7
+ Revision: 1362216
- Fix lld + annobin integration & Setup basic CI tests
- 7.0.1

* Mon Nov 26 2018 tv <tv> 7.0.0-1.mga7
+ Revision: 1334867
- fix build on !64bit
- BR python2-lit for testsuite
- "fix" underlinking
- further fix unittests linking
- fix unittests linking
- fix tools linking
- imported package lld


* Wed Nov 14 2018 Thierry Vignaud <tv@mageia.org> 7.0.0-1.mga7
- import from FC

* Mon Sep 24 2018 Tom Stellard <tstellar@redhat.com> - 7.0.0-1
- 7.0.1 Release

* Tue Sep 11 2018 Tom Stellard <tstellar@redhat.com> - 7.0.0-0.4.rc3
- 7.0.0-rc3 Release

* Fri Aug 31 2018 Tom Stellard <tstellar@redhat.com> - 7.0.0-0.3.rc2
- 7.0.0-rc2 Release

* Thu Aug 30 2018 Tom Stellard <tstellar@redhat.com> - 7.0.0-0.2.rc1
- Enable make check

* Mon Aug 13 2018 Tom Stellard <tstellar@redhat.com> - 7.0.0-0.1.rc1
- 7.0.0-rc1 Release

* Fri Jul 13 2018 Fedora Release Engineering <releng@fedoraproject.org> - 6.0.1-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_29_Mass_Rebuild

* Wed Jun 27 2018 Tom Stellard <tstellar@redhat.com> - 6.0.1-1
- 6.0.1 Release

* Fri May 11 2018 Tom Stellard <tstellar@redhat.com> - 6.0.1-0.1.rc1
- 6.0.1-rc1 Release

* Thu Mar 08 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-1
- 6.0.0 Release

* Tue Feb 13 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-0.3.rc2
- 6.0.0-rc2 Release

* Thu Feb 08 2018 Fedora Release Engineering <releng@fedoraproject.org> - 6.0.0-0.2.rc1
- Rebuilt for https://fedoraproject.org/wiki/Fedora_28_Mass_Rebuild

* Thu Jan 25 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-0.1.rc1
- 6.0.0-rc1 Release

* Thu Dec 21 2017 Tom Stellard <tstellar@redhat.com> - 5.0.1-1
- 5.0.1 Release

* Mon Sep 11 2017 Tom Stellard <tstellar@redhat.com> - 5.0.0-1
- 5.0.0 Release

* Thu Aug 03 2017 Fedora Release Engineering <releng@fedoraproject.org> - 4.0.1-4
- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Binutils_Mass_Rebuild

* Wed Jul 26 2017 Fedora Release Engineering <releng@fedoraproject.org> - 4.0.1-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Mass_Rebuild

* Thu Jul 06 2017 Tom Stellard <tstellar@redhat.com> - 4.0.1-2
- Backport r307092

* Tue Jul 04 2017 Tom Stellard <tstellar@redhat.com> - 4.0.1-1
- 4.0.1 Release

* Tue Jul 04 2017 Tom Stellard <tstellar@redhat.com> - 4.0.0-4
- Fix build without llvm-static

* Wed May 31 2017 Tom Stellard <tstellar@redhat.com> - 4.0.0-3
- Remove llvm-static dependency

* Mon May 15 2017 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 4.0.0-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_26_27_Mass_Rebuild

* Tue Mar 14 2017 Tom Stellard <tstellar@redhat.com> - 4.0.0-1
- lld 4.0.0 Final Release
